from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.test.tools.pcs_unittest import TestCase

class CheckCorosyncOffline(TestCase):
    """
    tested in:
        pcs.lib.test.test_env.PushCorosyncConfLiveNoQdeviceTest
        pcs.lib.commands.test.sbd.test_enable_sbd
    """

class DistributeCorosyncConf(TestCase):
    """
    tested in:
        pcs.lib.test.test_env.PushCorosyncConfLiveNoQdeviceTest
        pcs.lib.commands.test.sbd.test_enable_sbd
    """
