from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.lib.communication import qdevice_net
from pcs.test.tools.pcs_unittest import TestCase

class GetCaCert(TestCase):
    """
    tested in:
        pcs.lib.commands.test.test_quorum.AddDeviceNetTest
    """

class ClientSetup(TestCase):
    """
    tested in:
        pcs.lib.commands.test.test_quorum.AddDeviceNetTest
    """

class SignCertificate(TestCase):
    """
    tested in:
        pcs.lib.commands.test.test_quorum.AddDeviceNetTest
    """

class ClientImportCertificateAndKey(TestCase):
    """
    tested in:
        pcs.lib.commands.test.test_quorum.AddDeviceNetTest
    """

class ClientDestroy(TestCase):
    """
    tested in:
        pcs.lib.commands.test.test_quorum.RemoveDeviceNetTest
    """
